jQuery.sap.declare("sas.stress.wkbench.model.Process");
jQuery.sap.require("sas.stress.wkbench.StressAPI");
jQuery.sap.require("sas.stress.wkbench.model.Sheet");
jQuery.sap.require("sas.stress.wkbench.StressUtil");

sap.ui.base.Object.extend("sas.stress.wkbench.model.Process", {
    constructor: function (project, id, name, description, startId, endId) {
        this.project = project;
        this.id = id;
        this.name = name;
        this.description = description;
        this.startId = startId;
        this.endId = endId;
        this.tasks = {};
        this.transitions = {};
        this.transGroups = {};
        this.transGroupsBySource = {};
        this.plannedStartDates = {};
        this.plannedEndDates = {};
        this.startDate = sap.ui.core.format.DateFormat.getDateInstance().parse(project.getProcessStartDate()).getTime();
    }
});

sas.stress.wkbench.model.Process.prototype.getPlannedStartDate = function (taskId) {
    return this.plannedStartDates[taskId];
};
sas.stress.wkbench.model.Process.prototype.getPlannedEndDate = function (taskId) {
    return this.plannedEndDates[taskId];
};
sas.stress.wkbench.model.Process.prototype.getId = function () {
    return this.id;
};
sas.stress.wkbench.model.Process.prototype.getName = function () {
    return this.name;
};
sas.stress.wkbench.model.Process.prototype.getProject = function () {
    return this.project;
};
sas.stress.wkbench.model.Process.prototype.getProcessProject = function () {
    return this.project;
};
sas.stress.wkbench.model.Process.prototype.addTask = function (task) {
    this.tasks[task.getId()] = task;
    this.plannedEndDates[task.id] = task.plannedEndDate;
    this.plannedStartDates[task.id] = task.plannedStartDate;
};
sas.stress.wkbench.model.Process.prototype._getObj = function (obj, key, isArray) {
    var holder = obj[key];
    if (!holder) {
        holder = isArray ? [] : {};
        obj[key] = holder;
    }
    return holder;
};
sas.stress.wkbench.model.Process.prototype.addTransition = function (transition) {
    this.transitions[transition.getId()] = transition;
    // By Source ID
    var groupsBySource = this._getObj(this.transGroupsBySource, transition.getSourceId());
    var groupBySource = this._getObj(groupsBySource, transition.getSyncGroup(), true);
    groupBySource.push(transition);//eg:transGroups:{ sourceId:{sync1:[t1,t2,t3],sync2:[t4,t6]}, sourceId2:{...}}

    // By Target ID
    // TODO: would be nice to remove need for this second section (see getNodes and getLinks for usage of this style)
    var groupsByTarget = this._getObj(this.transGroups, transition.getTargetId());
    var groupByTarget = this._getObj(groupsByTarget, transition.getSyncGroup(), true);
    groupByTarget.push(transition);//eg:transGroups:{ sourceId:{sync1:[t1,t2,t3],sync2:[t4,t6]}, sourceId2:{...}}
};
sas.stress.wkbench.model.Process.prototype.getNodes = function () {

    var rb = sas.stress.wkbench.StressUtil.getBundle();
    var nodes = [];

    _.each(this.tasks, function (task) {

        var description = (task.getDescription() !== "") ? task.getDescription() : rb.getText("home.diagram.toolTip.description.missing.txt");

        nodes.push(
            {
                key: task.getId(),
                text: task.getName(),
                category: task.getType(),
                status: task.getStatusValue(),
                commentCount: task.getCommentCount(),
                taskGroup: task.getTaskGroup(),
                icon: task.getStatusIcon(),
                label: task.getStatusLabel(),
                description: task.getDescription(),
                toolTip: task.getName() + "\n\n" +
                         rb.getText("home.diagram.toolTip.description.fmt", [description]) + "\n" +
                         rb.getText("home.diagram.toolTip.duration.fmt", [task.getDueDays()])
            }
        );
        if (task.type === "START") {
            nodes.push({ key: task.getId() + "_START", text: rb.getText('task.activity.start.txt'), category: '_START_' });
        }
        if (task.type === "END") {
            nodes.push({ key: task.getId() + "_END", text: rb.getText('task.activity.end.txt'), category: '_END_' });
        }

    });
    _.each(this.transGroups, function (groups, targetId) {
        _.each(groups, function (ts, syncGroupId) {
            if (ts.length > 1) {
                nodes.push(
                    {
                        key: targetId + "_BAR",
                        text: '',
                        category: '_SYNCBAR_'
                    }
                );
            }
        });
    });

    return nodes;

};

sas.stress.wkbench.model.Process.prototype.getLinks = function () {
    var links = [];
    var syncBars = {};
    var that = this;
    _.each(this.transitions, function (transition) {
        var mustSync = that.transGroups[transition.getTargetId()][transition.getSyncGroup()].length > 1;
        links.push(
            {
                from: transition.getSourceId(),
                to: transition.getTargetId() + (mustSync ? "_BAR" : ""),
                text: transition.getExpression()
            }
        );
        if (mustSync) {
            syncBars[transition.getTargetId()] = true;
        }
    });
    _.each(syncBars, function (val, targetId) {
        links.push(
            {
                from: targetId + "_BAR",
                to: targetId
            }
        );
    });
    _.each(this.tasks, function (task) {
        if (task.getType() === 'START') {
            links.push({ from: task.getId() + "_START", to: task.getId() });
        } else if (task.getType() === 'END') {
            links.push({ from: task.getId(), to: task.getId() + "_END" });
        }
    });
    return links;
};

sas.stress.wkbench.model.Process.prototype.getTasks = function () {
    return this.tasks;
};