jQuery.sap.require("sas.hc.m.CheckBox");
jQuery.sap.require("sas.hc.m.DatePicker");
jQuery.sap.require("sas.hc.m.Input");
jQuery.sap.require("sas.hc.m.Label");
jQuery.sap.require("sas.hc.m.Select");
jQuery.sap.require("sas.hc.ui.layout.form.Form");
jQuery.sap.require("sas.hc.ui.layout.form.ResponsiveGridLayout");
jQuery.sap.require("sas.hc.ui.layout.form.FormContainer");
jQuery.sap.require("sas.hc.ui.layout.form.FormElement");
jQuery.sap.require("sas.hc.ui.table.VisibleRowCountMode");
jQuery.sap.require("sas.stress.wkbench.StressUtil");
jQuery.sap.require("sas.risk.common.widgets.HelpLabel");
jQuery.sap.require("sas.risk.common.widgets.SelectTable");

sap.ui.jsfragment("sas.stress.wkbench.view.EditProjectDialog", {

    createContent: function (controller) {
        var NO_VERSION = "__NO__VERSION__";
        var rb=sas.stress.wkbench.StressUtil.getBundle();

        var nameInput = new sas.hc.m.Input({
            id: this.createId("EditProjectDialog.nameInput"),
            value:"{/name}",
            type:sap.m.InputType.Text,
            disableValidation:false,
            maxLength:64,
            liveChange: $.proxy(controller.editProjectDialogNameInputLiveChange,controller)
        });
        var nameInputLabel = new sas.hc.m.Label({
            text: rb.getText("editProjectDialog.name.control.label"),
            required:true,
            labelFor: nameInput
        });

        var descriptionInput = new sas.hc.m.Input({
            value:"{/description}",
            liveChange: $.proxy(controller.toggleEditProjectDialogSaveButton, controller),
            maxLength:256
        });

        var descriptionInputLabel = new sas.hc.m.Label({
            text: rb.getText("editProjectDialog.description.control.label"),
            labelFor: descriptionInput
        });

        var processNotification = new sas.hc.m.CheckBox({
            id: this.createId("EditProjectDialog.processNotification"),
            text: rb.getText("editProjectDialog.processNotification.control.label"),
            selected: "{/processNotificationType}",
            select: $.proxy(controller.toggleEditProjectDialogSaveButton, controller),
            enabled: {
                path:"/projectIsLocked",
                formatter:function(isLocked){
                    if(isLocked===undefined||isLocked===null){
                        return true;
                    }
                    return !isLocked;
                }
            }
        });

        var processStartDatePicker = new sas.hc.m.DatePicker({
            id: this.createId("EditProjectDialog.processStartDatePicker"),
            value: {
                path: "/processStartDate",
                type: new sap.ui.model.type.Date({source: {pattern: "yyyy-MM-dd"}})
            },
            placeholder: "  ",
            change: $.proxy(controller.editProjectDialogProcessStartDateChange,controller),
            //liveChange: $.proxy(controller.editProjectDialogProcessStartDateLiveChange,controller),       //sas.hc.m.DatePicker does not support liveChange
            enabled: {
                path:"/projectIsLocked",
                formatter:function(isLocked){
                    if(isLocked===undefined||isLocked===null){
                        return false;
                    }
                    return !isLocked;
                }
            }
        });
        var processStartDatePickerLabel = new sas.risk.common.widgets.HelpLabel({
            id: this.createId("EditProjectDialog.processStartDatePickerLabel"),
            text: rb.getText("editProjectDialog.processStartDate.control.label"),
            labelFor: processStartDatePicker,
            library: "sas.stress",
            key: "newProjectDialog.processStartDate"
        });
        processStartDatePickerLabel.toggleStyleClass("ui-state-disabled", true);

        var processDefnItemTemplate = new sas.hc.ui.core.Item({key: "{id}", text:"{name}"});
        var processDefnNameSelect = new sas.hc.m.Select({
            id: this.createId("EditProjectDialog.processDefnNameSelect"),
            items: {
                path: "/definitions",
                template: processDefnItemTemplate
            },
            change: function() {
                var processDefn = parseInt(this.getSelectedItem().getKey());
                processNotification.setEnabled(processDefn > 0);
                processNotification.setSelected(false);
                processStartDatePicker.setEnabled(processDefn > 0);
                processStartDatePickerLabel.toggleStyleClass("ui-state-disabled", !processStartDatePicker.getEnabled());
                controller.toggleEditProjectDialogSaveButton();
            },
            enabled: {
                path:"/projectIsLocked",
                formatter:function(isLocked){
                    if(isLocked===undefined||isLocked===null){
                        return true;
                    }
                    return isLocked;
                }
            }
        });
        var processDefnLabel = new sas.hc.m.Label({
            text: rb.getText("editProjectDialog.processDefinitionName.control.label"),
            labelFor: processDefnNameSelect
        });

        var asOfDatePicker = new sas.hc.m.DatePicker({
            id: this.createId("EditProjectDialog.asOfDatePicker"),
            value: {
                path: "/asOfDate",
                type: new sap.ui.model.type.Date({source: {pattern: "yyyy-MM-dd"}})
            },
            placeholder: "  ",
            change: $.proxy(controller.editProjectDialogAsOfDateChange,controller),
            enabled: {
                path:"/projectIsLocked",
                formatter:function(isLocked){
                    if(isLocked===undefined||isLocked===null){
                        return true;
                    }
                    return isLocked;
                }
            }
            //liveChange: $.proxy(controller.editProjectDialogAsOfDateLiveChange,controller)      //sas.hc.m.DatePicker does not support liveChange
        });
        var asOfDateLabel = new sas.risk.common.widgets.HelpLabel({
            text: rb.getText("editProjectDialog.asOfDate.control.label"),
            required:true,
            labelFor:asOfDatePicker,
            library: "sas.stress",
            key: "newProjectDialog.asOfDate"
        });

        this.workSheetList = new sas.risk.common.widgets.SelectTable({
            id: this.createId("EditProjectDialog.workSheetList"),
            nameAttribute: "nameVersion",
            listItems: "{/worksheets}",
            listName:  rb.getText("project.worksheets.name.txt"),
            visibleRowCountMode: sas.hc.ui.table.VisibleRowCountMode.Auto,
            minAutoRowCount: 5,            //visibleRowCount: 4,
            //columnHeaderVisible: true,
            layoutData: new sas.hc.m.FlexItemData({ growFactor: 2 }),
            selectionChange: $.proxy(controller.toggleEditProjectDialogSaveButton, controller)
        });

        var worksheetVersion = new sas.hc.m.Text({wrapping: false}).bindProperty("text", {
            path: "version",
            formatter: function (version) {
                return (version === NO_VERSION) ? "" : version ;
            }
        });

        this.workSheetList.addColumn(new sas.hc.ui.table.Column({
            label: new sas.hc.m.Label({ text: rb.getText("project.worksheets.worksheetVersion.txt") }),
            template: worksheetVersion,
            sortProperty: "version"
        }));

        this.workSheetList.bindRows("/worksheets");
        //S1477186 Disabling project dimensions story
        // this.dimensionList = new sas.risk.common.widgets.SelectTable({
        //     id: this.createId("EditProjectDialog.dimensionList"),
        //     nameAttribute: "name",
        //     listItems: "{/dimensionKeys}",
        //     listName:  rb.getText("project.dimensionList.name.txt"),
        //     visibleRowCountMode: sas.hc.ui.table.VisibleRowCountMode.Auto,
        //     minAutoRowCount: 4,            //visibleRowCount: 4,
        //     //columnHeaderVisible: true,
        //     layoutData: new sas.hc.m.FlexItemData({ growFactor: 2 }),
        //     selectionChange: $.proxy(controller.toggleEditProjectDialogSaveButton, controller)
        // });
        //
        // this.dimensionList.bindRows("/dimensionKeys");

        var requireComments=new sas.hc.m.CheckBox({
            id: this.createId("EditProjectDialog.requireComments"),
            text: rb.getText("editProjectDialog.require.comments.control.label"),
            selected: "{/requireComments}",
            select: $.proxy(controller.toggleEditProjectDialogSaveButton, controller),
            enabled: {
                path:"/projectIsLocked",
                formatter:function(isLocked){
                    if(isLocked===undefined||isLocked===null){
                        return true;
                    }
                    return isLocked;    // this is the opposite
                }
            }
        });//.addStyleClass("sapUiTinyMargin");



        var dimensionCrossingFileUploader = new sas.hc.ui.unified.FileUploader({
            id: this.createId("EditProjectDialog.dimensionCrossingFileUploader"),
            name: "file",
            uploadUrl: "",
            uploadOnChange: false,
            sendXHR: true, //using this to get the response in a better format.
            fileType: "csv,xls,xlsx",
            placeholder: rb.getText("EditProjectDialog.fileUploader.placeholder.txt"),
            value: "{/dimensionCrossingName}",
            maximumFileSize: 100, //100MB
            uploadComplete: $.proxy(controller.editDimensionCrossingFileUploadComplete, controller),
            typeMissmatch: $.proxy(controller.editDimensionCrossingFileUploadTypeMissmatch, controller),
            fileSizeExceed: $.proxy(controller.editDimensionCrossingFileUploadFileSizeExceed, controller),
            change: function (evt) {
                var fileSelected = (dimensionCrossingFileUploader.getValue() !== "");
                // S1243360: Workaround for Chrome-specific issue inherited from SAP. When user hits "Cancel" after having
                // already made a file selection the file is cleared but the UI still shows the filename; we need to clear
                // the filename displayed in the UI so it is apparent to the user that no file is selected anymore.
                if (!fileSelected) {
                    dimensionCrossingFileUploader.getDomRef("fu_input-inner").value = "";
                }
                controller.toggleEditProjectDialogSaveButton(evt);
            },
            layoutData: new sas.hc.m.FlexItemData({
                growFactor: 0
            }),
            width:"100%"
        });

        var dimensionCrossingFileUploaderLabel = new sas.hc.m.Label({
            text: rb.getText("project.dimensionCrossingFile.label"),
            labelFor: this.dimensionCrossingFileUploader
            //required: true           // worksheet is not required per Jeff
        }).addStyleClass("sapUiTinyMarginTop")/*.addStyleClass("sapUiTinyMarginBegin")*/;

        var clearButton = new sas.hc.m.Button({
            id: this.createId("EditProjectDialog.clearButton"),
            //hasPopup: true,
            //icon: sas.icons.HC.ERRORSTATUS,
            text: rb.getText("editProjectDialog.clearButton.control.label"),
            enabled: {
             path: "/dimensionCrossingName",
               formatter: function (dimensionCrossingName) {
                   return (!(dimensionCrossingName === "" || dimensionCrossingName == undefined));
               }
            },
            //"{/dimensionCrossingName}",
            press: $.proxy(controller.clearDimensionCrossingNameButtonPress, controller),
            layoutData: new sas.hc.m.FlexItemData({
                growFactor: 10
            })
        });
        var runTimeLayout=new sas.hc.m.HBox/*.HorizontalLayout*/({
            items:[
                dimensionCrossingFileUploader,
                clearButton
            ],
            width:"100%"
        });

        var form = new sas.hc.ui.layout.form.Form({
            layout: new sas.hc.ui.layout.form.ResponsiveGridLayout({
                columnsM: 2,
                columnsL: 2,
                labelSpanS: 12,
                labelSpanM: 12,
                labelSpanL: 12
            }),
            formContainers: [
                new sas.hc.ui.layout.form.FormContainer("FormContainer", {
                    formElements: [
                        new sas.hc.ui.layout.form.FormElement({
                            label: nameInputLabel,
                            fields: [nameInput]
                        }),
                        new sas.hc.ui.layout.form.FormElement({
                            label: processDefnLabel,
                            fields: [processDefnNameSelect]
                        }),
                        new sas.hc.ui.layout.form.FormElement({
                            label: processStartDatePickerLabel,
                            fields: [processStartDatePicker]
                        })
                    ]
                }),
                new sas.hc.ui.layout.form.FormContainer("FormContainer2", {
                    formElements: [
                        new sas.hc.ui.layout.form.FormElement({
                            label: descriptionInputLabel,
                            fields: [descriptionInput]
                        }),
                        new sas.hc.ui.layout.form.FormElement({
                            label: "",
                            fields: [processNotification]
                        }),
                        new sas.hc.ui.layout.form.FormElement({
                            label: asOfDateLabel,
                            fields: [asOfDatePicker]
                        })
                    ]
                })
            ]
        });
        //form.addStyleClass("stressFormLarge");

        // the dimension and sheets list go into a form so that resizing does not
        // cause issues. -- SRINI S1385107
        var dimensionsAndSheetsForm = new sas.hc.ui.layout.form.Form({
            layout: new sas.hc.ui.layout.form.ResponsiveGridLayout({
                columnsM: 1,
                columnsL: 1,
                labelSpanS: 12,
                labelSpanM: 12,
                labelSpanL: 12
            }),
            formContainers: [
                new sas.hc.ui.layout.form.FormContainer("DimensionsAndSheetsContainer", {
                    formElements: [
                        //S1477186 Disabling project dimensions story
                        // new sas.hc.ui.layout.form.FormElement({
                        //     label:new sas.hc.m.Label({
                        //         text: rb.getText("project.dimensionList.label"),
                        //         labelFor: this.dimensionList
                        //         //required: true           // worksheet is not required per Jeff
                        //     }),
                        //     fields: [this.dimensionList]
                        // }),
                        new sas.hc.ui.layout.form.FormElement({
                            label:new sas.hc.m.Label({
                                text: rb.getText("project.worksheets.label"),
                                labelFor: this.workSheetList
                                //required: true           // worksheet is not required per Jeff
                            }).addStyleClass("sapUiSmallMarginTop"),
                            fields:[this.workSheetList]
                        })
                    ]
                })
            ]
        });

        var form2 = new sas.hc.ui.layout.form.Form({
            layout: new sas.hc.ui.layout.form.ResponsiveGridLayout({
                columnsM: 2,
                columnsL: 2,
                labelSpanS: 12,
                labelSpanM: 12,
                labelSpanL: 12
            }),
            formContainers: [
                new sas.hc.ui.layout.form.FormContainer("FormContainer3", {
                    formElements: [
                        new sas.hc.ui.layout.form.FormElement({
                            fields: [ requireComments]
                        })
                    ]
                }),
                new sas.hc.ui.layout.form.FormContainer("FormContainer4", {
                    formElements: [
                        new sas.hc.ui.layout.form.FormElement({
                            label: dimensionCrossingFileUploaderLabel,
                            fields: [runTimeLayout]
                        })
                    ]
                })
            ]
        });

        // use Another Form to place all the forms into. Using a VBox has issues
        // when resized because the VBox takes up empty spaces to fit all the contents
        // this makes some of the controls creep into spaces that are used by other
        // controls. because all the controls are now in a Form layout the form layout
        // makes sure items are in right locations always.  -- SRINI S1385107
        var container = new sas.hc.ui.layout.form.Form({
            layout: new sas.hc.ui.layout.form.ResponsiveGridLayout({
                columnsM: 1,
                columnsL: 1,
                labelSpanS: 12,
                labelSpanM: 12,
                labelSpanL: 12
            }),
            formContainers: [
                new sas.hc.ui.layout.form.FormContainer("MainFormContainer", {
                    formElements: [
                        new sas.hc.ui.layout.form.FormElement({
                            fields: [form]
                        }),
                        new sas.hc.ui.layout.form.FormElement({
                            fields:[dimensionsAndSheetsForm]
                        }),
                        new sas.hc.ui.layout.form.FormElement({
                            fields:[form2]
                        })
                    ]
                })
            ]
        });

        var dialog = new sas.hc.m.Dialog({
            id: "sas.stress.wkbench.view.EditProjectDialog",
            title: "{/title}", //rb.getText("editProjectDialog.window.title"),
            initialFocus: nameInput,
            content: [container],
            /*contentWidth: "55rem", S1387014
            contentHeight: "100%",*/
            beginButton: new sas.hc.m.Button({
                id: this.createId("EditProjectDialog.saveProjectButton"),
                text: rb.getText("button.save.action.label"),
                enabled: false,
                press: $.proxy(controller.handleProjectDialogSaveButtonPress, controller)
            }),
            endButton: new sas.hc.m.Button({
                text: rb.getText("button.cancel.action.label"),
                press: function () {
                    dialog.close();
                }
            }),
            afterClose: function () {
                dialog.destroy();
            }
        });

        return dialog;
    }
});